@@ -1,6 +1,6 @@
 /*=============================================================================
 ttyrpld - TTY replay daemon
-freebsd-5.3/rpldev.c - Stage 1 RPL interface
+freebsd-5.3/rpldev.c - Kernel interface for RPLD
   Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004 - 2005
   -- License restrictions apply (GPL v2)
 
@@ -25,23 +25,26 @@ freebsd-5.3/rpldev.c - Stage 1 RPL interface
 #include <sys/conf.h>
 #include <sys/endian.h>
 #include <sys/errno.h>
+#include <sys/ioccom.h>
 #include <sys/kernel.h>
 #include <sys/malloc.h>
 #include <sys/module.h>
 #include <sys/mutex.h>
 #include <sys/proc.h>
+#include <sys/syslog.h>
 #include <sys/systm.h>
+#include <sys/time.h>
 #include <sys/tty.h>
 #include <sys/types.h>
 #include <sys/uio.h>
-#include <sys/rpl.h>
+#include <sys/km_rpldev.h>
 #include "../include/rpl_ioctl.h"
 #include "../include/rpl_packet.h"
 
-#define IO_NDELAY 0x10 // sys/vnode.h -- inclusion problems
-#define PTY_MASTER(tty) (major(tty->t_dev) == 6)
+#define IO_NDELAY      0x10 // from sys/vnode.h -- inclusion problems
+#define SKIP_PTM(tty)  if(major(tty->t_dev) == 6) { return 0; }
 #define TTY_DEVNR(tty) htole32(mkdev_26(major(tty->t_dev), minor(tty->t_dev)))
-#define RET_IF_INACT if(!kmi_usecount) { return 0; }
+#define RET_IF_INACT   if(!kmi_usecount) { return 0; }
 
 // Module stuff
 static int kmd_event(module_t, int, void *);
@@ -53,21 +56,23 @@ static int kio_init(struct cdev *, struct tty *);
 static int kio_open(struct cdev *, struct tty *);
 static int kio_read(const char *, int, struct tty *);
 static int kio_write(const char *, int, struct tty *);
-static int kio_ioctl(struct tty *, u_long, void *);
+//static int kio_ioctl(struct tty *, u_long, void *);
 static int kio_close(struct tty *);
 static int kio_deinit(struct tty *);
 
 // Stage 3 functions
 static int uio_open(struct cdev *, int, int, struct thread *);
 static int uio_read(struct cdev *, struct uio *, int);
+static int uio_ioctl(struct cdev *, u_long, caddr_t, int, struct thread *);
 static int uio_close(struct cdev *, int, int, struct thread *);
 
 // Local functions
 inline static ssize_t avail_R(void);
 inline static ssize_t avail_W(void);
+inline static void fill_time(struct timeval *);
 inline static unsigned int min_uint(unsigned int, unsigned int);
 inline static uint32_t mkdev_26(unsigned long, unsigned long);
-static int mv_buffer(struct rpld_packet *, const void *, size_t);
+static int mv_buffer(struct rpldev_packet *, const void *, size_t);
 inline static void mv_buffer2(const void *, size_t);
 inline static int mv_to_user(struct uio *, size_t);
 
@@ -75,10 +80,10 @@ inline static int mv_to_user(struct uio *, size_t);
 static MALLOC_DEFINE(Buffer_malloc, "rpldev", "rpldev ring buffer");
 static struct mtx Buffer_lock, Open_lock;
 static char *Buffer = NULL, *BufRP = NULL, *BufWP = NULL;
-static size_t Bufsize = 32768;
-static int Open_count = 0, Pull_queue;
+static size_t Bufsize = 32 * 1024;
+static int Open_count = 0;
 
-// Kernel module info (kmi) stuff
+// Kernel module info stuff
 static int kmi_usecount = 0;
 static struct cdev *kmi_node;
 static struct cdevsw kmi_fops = {
@@ -87,6 +92,7 @@ static struct cdevsw kmi_fops = {
     .d_maj     = MAJOR_AUTO,
     .d_open    = uio_open,
     .d_read    = uio_read,
+    .d_ioctl   = uio_ioctl,
     .d_close   = uio_close,
 };
 static moduledata_t kmi_rpldev = {
@@ -103,10 +109,8 @@ static int kmd_event(module_t mp, int type, void *data) {
         case MOD_LOAD:
             if((eax = kmd_load()) != 0) { kmd_unload(); }
             return eax;
-            break;
         case MOD_UNLOAD:
             return kmd_unload();
-            break;
     }
     return EOPNOTSUPP;
 }
@@ -130,95 +134,106 @@ static int kmd_unload(void) {
 
 //-----------------------------------------------------------------------------
 static int kio_init(struct cdev *cd, struct tty *tty) {
-    struct rpld_packet p;
+    struct rpldev_packet p;
 
     RET_IF_INACT;
-    if(PTY_MASTER(tty)) { return 0; }
+    SKIP_PTM(tty);
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = 0;
     p.event = EVT_INIT;
     p.magic = MAGIC_SIG;
+    fill_time(&p.time);
     return mv_buffer(&p, NULL, 0);
 }
 
 static int kio_open(struct cdev *cd, struct tty *tty) {
-    struct rpld_packet p;
+    struct rpldev_packet p;
 
     RET_IF_INACT;
-    if(PTY_MASTER(tty)) { return 0; }
+    SKIP_PTM(tty);
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = 0;
     p.event = EVT_OPEN;
     p.magic = MAGIC_SIG;
+    fill_time(&p.time);
     return mv_buffer(&p, NULL, 0);
 }
 
 static int kio_read(const char *buf, int count, struct tty *tty) {
-    struct rpld_packet p;
+    struct rpldev_packet p;
 
     RET_IF_INACT;
-    if(count == 0 || PTY_MASTER(tty)) { return 0; }
+    SKIP_PTM(tty);
+    if(count == 0) { return 0; }
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = htole16(count);
     p.event = EVT_READ;
     p.magic = MAGIC_SIG;
+    fill_time(&p.time);
     return mv_buffer(&p, buf, count);
 }
 
 static int kio_write(const char *buf, int count, struct tty *tty) {
-    struct rpld_packet p;
+    struct rpldev_packet p;
 
     RET_IF_INACT;
-    if(count == 0 || PTY_MASTER(tty)) { return 0; }
+    SKIP_PTM(tty);
+    if(count == 0) { return 0; }
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = htole16(count);
     p.event = EVT_WRITE;
     p.magic = MAGIC_SIG;
+    fill_time(&p.time);
     return mv_buffer(&p, buf, count);
 }
 
+/*
 static int kio_ioctl(struct tty *tty, u_long cmd, void *arg) {
-    struct rpld_packet p;
+    struct rpldev_packet p;
     uint32_t cmd32;
 
     RET_IF_INACT;
-    if(PTY_MASTER(tty)) { return 0; }
+    SKIP_PTM(tty);
 
     cmd32   = cmd;
     p.dev   = TTY_DEVNR(tty);
     p.size  = htole16(sizeof(cmd32));
     p.event = EVT_IOCTL;
     p.magic = MAGIC_SIG;
+    fill_time(&p.time);
     return mv_buffer(&p, &cmd32, sizeof(cmd32));
 }
+*/
 
 static int kio_close(struct tty *tty) {
-    struct rpld_packet p;
+    struct rpldev_packet p;
 
     RET_IF_INACT;
-    if(PTY_MASTER(tty)) { return 0; }
+    SKIP_PTM(tty);
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = 0;
     p.event = EVT_CLOSE;
     p.magic = MAGIC_SIG;
+    fill_time(&p.time);
     return mv_buffer(&p, NULL, 0);
 }
 
 static int kio_deinit(struct tty *tty) {
-    struct rpld_packet p;
+    struct rpldev_packet p;
 
     RET_IF_INACT;
-    if(PTY_MASTER(tty)) { return 0; }
+    SKIP_PTM(tty);
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = 0;
     p.event = EVT_DEINIT;
     p.magic = MAGIC_SIG;
+    fill_time(&p.time);
     return mv_buffer(&p, NULL, 0);
 }
 
@@ -237,6 +252,7 @@ static int uio_open(struct cdev *cd, int flag, int mode, struct thread *th) {
     if((Buffer = malloc(Bufsize, Buffer_malloc, M_WAITOK)) == NULL) {
         mtx_unlock(&Buffer_lock);
         --kmi_usecount;
+        --Open_count;
         return ENOMEM;
     }
 
@@ -245,7 +261,10 @@ static int uio_open(struct cdev *cd, int flag, int mode, struct thread *th) {
     rpl_qopen   = kio_open;
     rpl_qread   = kio_read;
     rpl_qwrite  = kio_write;
-    rpl_qioctl  = kio_ioctl;
+    /* ioctl reporting is deactivated, because 1) a _lot_ more ioctls are
+    generated in *BSD compared to Linux, 2) ioctls are not yet interpreted by
+    userspace apps (ttyreplay). */
+    //rpl_qioctl  = kio_ioctl;
     rpl_qclose  = kio_close;
     rpl_qdeinit = kio_deinit;
     mtx_unlock(&Buffer_lock);
@@ -260,17 +279,60 @@ static int uio_read(struct cdev *cd, struct uio *uio, int flags) {
     while(BufRP == BufWP) {
         mtx_unlock(&Buffer_lock);
         if(flags & IO_NDELAY) { return EWOULDBLOCK; }
-        if((eax = tsleep(&Pull_queue, PCATCH, "rpldev", 0)) != 0) {
+        if((eax = tsleep(&Buffer, PCATCH, "rpldev", 0)) != 0) {
             return eax;
         }
         mtx_lock(&Buffer_lock);
     }
+
     count = min_uint(uio->uio_resid, avail_R());
     eax = mv_to_user(uio, count);
     mtx_unlock(&Buffer_lock);
     return eax;
 }
 
+static int uio_ioctl(struct cdev *cd, u_long cmd, caddr_t data, int flags,
+ struct thread *th)
+{
+    /* Gah, it does not even support multiple return values >:-( so I need to
+    hack around with data. */
+    size_t *ptr = (void *)data;
+    if(IOCGROUP(cmd) != RPL_IOC_MAGIC) { return ENOTTY; }
+    if(ptr == NULL) { return EFAULT; }
+
+    switch(cmd) {
+        case RPL_IOC_GETBUFSIZE:
+            *ptr = Bufsize;
+            return 0;
+        case RPL_IOC_GETRAVAIL:
+            *ptr = avail_R();
+            return 0;
+        case RPL_IOC_GETWAVAIL:
+            *ptr = avail_W();
+            return 0;
+        case RPL_IOC_IDENTIFY:
+            *ptr = 0xC0FFEE;
+            return 0;
+        case RPL_IOC_SEEK:
+            mtx_lock(&Buffer_lock);
+            BufRP = Buffer + (BufRP - Buffer + min_uint(*ptr,
+              avail_R())) % Bufsize;
+            if(BufRP >= Buffer + Bufsize) { 
+                log(LOG_ERR, "BufRP overflow (should not happen!) -"
+                 " resetting BufRP/BufWP\n");
+                BufRP = BufWP = 0;
+            }
+            mtx_unlock(&Buffer_lock);
+            return 0;
+        case RPL_IOC_FLUSH:
+            mtx_lock(&Buffer_lock);
+            BufRP = BufWP;
+            mtx_unlock(&Buffer_lock);
+            return 0;
+    }
+    return ENOTTY;
+}
+
 static int uio_close(struct cdev *cd, int flags, int fmt, struct thread *th) {
     rpl_qinit   = NULL;
     rpl_qopen   = NULL;
@@ -298,6 +360,23 @@ inline static ssize_t avail_W(void) {
     return BufRP - BufWP - 1;
 }
 
+inline static void fill_time(struct timeval *tv) {
+    microtime(tv);
+
+    if(sizeof(tv->tv_sec) == sizeof(uint32_t)) {
+        tv->tv_sec = htole32(tv->tv_sec);
+    } else if(sizeof(tv->tv_sec) == sizeof(uint64_t)) {
+        tv->tv_sec = htole64(tv->tv_sec);
+    }
+
+    if(sizeof(tv->tv_usec) == sizeof(uint32_t)) {
+        tv->tv_usec = htole32(tv->tv_usec);
+    } else if(sizeof(tv->tv_usec) == sizeof(uint64_t)) {
+        tv->tv_usec = htole64(tv->tv_usec);
+    }
+    return;
+}
+
 inline static unsigned int min_uint(unsigned int a, unsigned int b) {
     return a < b ? a : b;
 }
@@ -306,17 +385,17 @@ inline static uint32_t mkdev_26(unsigned long maj, unsigned long min) {
     return (maj << 20) | (min & 0xFFFFF);
 }
 
-static int mv_buffer(struct rpld_packet *p, const void *buf, size_t count) {
+static int mv_buffer(struct rpldev_packet *p, const void *buf, size_t count) {
     mtx_lock(&Buffer_lock);
-    if(avail_W() < sizeof(struct rpld_packet) + count) {
+    if(avail_W() < sizeof(struct rpldev_packet) + count) {
         mtx_unlock(&Buffer_lock);
         return ENOSPC;
     }
 
-    mv_buffer2(p, sizeof(struct rpld_packet));
+    mv_buffer2(p, sizeof(struct rpldev_packet));
     if(count > 0) { mv_buffer2(buf, count); }
     mtx_unlock(&Buffer_lock);
-    wakeup_one(&Pull_queue);
+    wakeup_one(&Buffer);
     return count;
 }
 
