@@ -32,16 +32,25 @@ include/dev.h - Supply file for dev_t encoding
     (((major) << K26_MINORBITS) | ((minor) & K26_MINORMASK))
 #define KD26_PARTS(dev) K26_MAJOR(dev), K26_MINOR(dev)
 
+#if defined(__FreeBSD__) || defined(__OpenBSD__)
+// Another mix
+#    define COMPAT_MAJOR(dev) \
+        (unsigned long)(((dev) & 0xFF00) >> 8)
+#    define COMPAT_MINOR(dev) \
+        (unsigned long)(((dev) & 0xFF) | (((dev) & ~0xFFFF) >> 16))
+#    define COMPAT_MKDEV(major, minor) \
+        (unsigned long)(((minor) & 0xFF) | (((major) & 0xFF) << 8) | \
+        (((minor) & ~0xFF) << 16))
+#else
 // Glibc uses an intermediate mixture to be 2.4-compatible
-
-#define COMPAT_MINORBITS 8
-#define COMPAT_MAJOR(dev) \
-    (unsigned long)(((dev) & 0xFFF00) >> COMPAT_MINORBITS)
-#define COMPAT_MINOR(dev) \
-    (unsigned long)(((dev) & 0xFF) | (((dev) & 0xFFF00000) >> 20))
-#define COMPAT_MKDEV(major, minor) \
-    (((minor) & 0xFF) | (((minor) & 0xFFF00) << 20) | \
-    (((major) & 0xFFF) << COMPAT_MINORBITS))
+#    define COMPAT_MAJOR(dev) \
+        (unsigned long)(((dev) & 0xFFF00) >> 8)
+#    define COMPAT_MINOR(dev) \
+        (unsigned long)(((dev) & 0xFF) | (((dev) & 0xFFF00000) >> 20))
+#    define COMPAT_MKDEV(major, minor) \
+        (unsigned long)(((minor) & 0xFF) | (((minor) & 0xFFF00) << 20) | \
+        (((major) & 0xFFF) << 8))
+#endif
 
 #endif // RPL_DEV_H
 
