@@ -35,11 +35,16 @@ include/rpl_ioctl.h - IOCTL numbers for RPLDEV
 #endif
 
 enum {
+    /* Some weird *BSD scheme makes me use IOWR, otherwise readin the pointer
+    within kernel space does not seem to work. */
     RPL_IOC_MAGIC      = 0xB7,
-    RPL_IOC_GETBUFSIZE = _IOR(RPL_IOC_MAGIC, 1, size_t),
-    RPL_IOC_GETRAVAIL  = _IOR(RPL_IOC_MAGIC, 2, size_t),
-    RPL_IOC_GETWAVAIL  = _IOR(RPL_IOC_MAGIC, 3, size_t),
-    RPL_IOC__MAXNUM    = _IO(RPL_IOC_MAGIC, 4),
+    RPL_IOC_GETBUFSIZE = _IOWR(RPL_IOC_MAGIC, 1, size_t),
+    RPL_IOC_GETRAVAIL  = _IOWR(RPL_IOC_MAGIC, 2, size_t),
+    RPL_IOC_GETWAVAIL  = _IOWR(RPL_IOC_MAGIC, 3, size_t),
+    RPL_IOC_IDENTIFY   = _IOWR(RPL_IOC_MAGIC, 4, size_t),
+    RPL_IOC_SEEK       = _IOWR(RPL_IOC_MAGIC, 5, size_t),
+    RPL_IOC_FLUSH      = _IOWR(RPL_IOC_MAGIC, 6, size_t),
+    RPL_IOC__MAXNUM    = _IOWR(RPL_IOC_MAGIC, 7, size_t),
 };
 
 #endif // RPL_IOCTL_H
