@@ -27,44 +27,42 @@ include/rpl_packet.h - Packet definitions for RPLD and RPLDEV
 #define __PACKED __attribute__((packed))
 
 #if defined(__linux__) && defined(__KERNEL__)
+#    include <linux/time.h>
 #    include <linux/types.h>
-#elif defined(__FreeBSD__) && defined(_KERNEL)
+#elif (defined(__FreeBSD__) || defined(__OpenBSD__)) && defined(_KERNEL)
+#    include <sys/time.h>
 #    include <sys/types.h>
 #else // userspace
-#    include <stdint.h>
+#    include <sys/time.h>
+#    include "rpl_stdint.h"
 #endif
 
 enum {
-    MAGIC_SIG = 0xEE,
     EVT_NONE = 0,
     EVT_OPEN,
     EVT_READ,
     EVT_WRITE,
     EVT_IOCTL,
     EVT_CLOSE,
-    EVT_INIT   = 'i',
-    EVT_DEINIT = 'd',
-    EVT_IDENT  = 240,
+    EVT_INIT    = 'i',
+    EVT_DEINIT  = 'd',
+    EVT_ID_PROG = 240,
+    EVT_ID_DEVPATH,
     EVT_max,
+    MAGIC_SIG = 0xEE,
 };
 
-struct rpld_packet {
+struct rpldev_packet {
     uint32_t dev;
     uint16_t size;
     uint8_t event, magic;
+    struct timeval time;
 } __PACKED;
 
-struct mem_packet {
-    uint32_t dev;
+struct rpldsk_packet {
     uint16_t size;
     uint8_t event, magic;
-    struct timeval tv;
-} __PACKED;
-
-struct disk_packet {
-    uint16_t size;
-    uint8_t event, magic;
-    struct timeval tv;
+    struct timeval time;
 } __PACKED;
 
 #endif // RPL_PACKET_H
