@@ -28,12 +28,14 @@ kernel/rpldev.c - Kernel interface for RPLD
 #include <linux/fs.h>
 #include <linux/init.h>
 #include <linux/kernel.h>
+#include <linux/km_rpldev.h>
 #include <linux/miscdevice.h>
 #include <linux/module.h>
 #include <linux/moduleparam.h>
 #include <linux/poll.h>
-#include <linux/rpl.h>
 #include <linux/string.h>
+#include <linux/time.h>
+#include <linux/tty.h>
 #include <linux/wait.h>
 #include <linux/version.h>
 #include <linux/vmalloc.h>
@@ -44,48 +46,62 @@ kernel/rpldev.c - Kernel interface for RPLD
 #    define __user
 #endif
 
+#define PREFIX "rpldev: "
+#define MAXFNLEN 256
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
-#    define KERNEL_2_4 1 // just for device number handling
-#    define PTY_MASTER(tty) \
-      (((tty)->driver->major >= UNIX98_PTY_MASTER_MAJOR && \
-      (tty)->driver->major < UNIX98_PTY_MASTER_MAJOR + \
-      UNIX98_PTY_MAJOR_COUNT) || (tty)->driver->major == PTY_MASTER_MAJOR)
+#    define KERNEL_2_4 1
+#    define IS_PTY_MASTER(tty) \
+      (((tty)->driver.major >= UNIX98_PTY_MASTER_MAJOR && \
+      (tty)->driver.major < UNIX98_PTY_MASTER_MAJOR + \
+      UNIX98_PTY_MAJOR_COUNT) || (tty)->driver.major == PTY_MASTER_MAJOR)
 #    define TTY_DEVNR(tty) \
       cpu_to_le32(mkdev_26(MAJOR(tty->device), MINOR(tty->device)))
 #else
-#    define PTY_MASTER(tty) \
+#    define IS_PTY_MASTER(tty) \
       ((tty)->driver->major == UNIX98_PTY_MASTER_MAJOR || \
       (tty)->driver->major == PTY_MASTER_MAJOR)
 #    define TTY_DEVNR(tty) \
       cpu_to_le32(mkdev_26(tty->driver->major, \
       tty->driver->minor_start + tty->index))
 #endif
+#ifndef SEEK_SET
+#    define SEEK_SET 0
+#endif
+#ifndef SEEK_CUR
+#    define SEEK_CUR 1
+#endif
+#ifndef SEEK_END
+#    define SEEK_END 2
+#endif
+#define SKIP_PTM(tty) if(IS_PTY_MASTER(tty)) { return 0; }
 
 // Stage 2 functions
-static int kio_init(struct tty_struct *, struct tty_struct *);
-static int kio_open(struct tty_struct *, struct tty_struct *);
+static int kio_init(struct tty_struct *, struct tty_struct *, struct file *);
+static int kio_open(struct tty_struct *, struct tty_struct *, struct file *);
 static int kio_read(const char __user *, size_t, struct tty_struct *);
 static int kio_write(const char __user *, size_t, struct tty_struct *);
+/*static int kio_ioctl(struct tty_struct *, struct tty_struct *, unsigned int,
+  unsigned long);*/
 static int kio_close(struct tty_struct *, struct tty_struct *);
 static int kio_deinit(struct tty_struct *, struct tty_struct *);
-static int kio_ioctl(struct tty_struct *, struct tty_struct *, unsigned int,
-  unsigned long);
 
 // Stage 3 functions
 static int uio_open(struct inode *, struct file *);
 static ssize_t uio_read(struct file *, char __user *, size_t, loff_t *);
-static loff_t uio_llseek(struct file *, loff_t, int);
+static loff_t uio_seek(struct file *, loff_t, int);
 static int uio_ioctl(struct inode *, struct file *, unsigned int,
- unsigned long);
+  unsigned long);
 static unsigned int uio_poll(struct file *, poll_table *);
 static int uio_close(struct inode *, struct file *);
 
 // Local functions
 inline static ssize_t avail_R(void);
 inline static ssize_t avail_W(void);
-inline static unsigned int imin_uint(unsigned int, unsigned int);
+inline static void fill_time(struct timeval *);
+inline static unsigned int min_uint(unsigned int, unsigned int);
 inline static uint32_t mkdev_26(unsigned long, unsigned long);
-static int mv_buffer(struct rpld_packet *, const void *, size_t);
+static int mv_buffer(struct rpldev_packet *, const void *, size_t);
 inline static void mv_buffer2(const void *, size_t);
 inline static void mv_to_user(char __user *, size_t);
 
@@ -94,14 +110,14 @@ static DECLARE_WAIT_QUEUE_HEAD(Pull_queue);
 static DECLARE_MUTEX(Buffer_lock);
 static DECLARE_MUTEX(Open_lock);
 static char *Buffer = NULL, *BufRP = NULL, *BufWP = NULL;
-static size_t Bufsize = 32768;
+static size_t Bufsize = 32 * 1024;
 static int Minor_nr = MISC_DYNAMIC_MINOR, Open_count = 0;
 
 // Kernel module info (kmi) stuff
 static struct file_operations kmi_fops = {
     .open    = uio_open,
     .read    = uio_read,
-    .llseek  = uio_llseek,
+    .llseek  = uio_seek,
     .ioctl   = uio_ioctl,
     .poll    = uio_poll,
     .release = uio_close,
@@ -113,12 +129,12 @@ static struct miscdevice kmi_miscinfo = {
     .fops  = &kmi_fops,
 };
 
-extern int __init init_module(void);
-extern void __exit cleanup_module(void);
+extern __init int init_module(void);
+extern __exit void cleanup_module(void);
 MODULE_DESCRIPTION("ttyrpld Kernel interface");
 MODULE_AUTHOR("Jan Engelhardt <jengelh [at] linux01 gwdg de>");
 MODULE_LICENSE("GPL v2");
-MODULE_PARM_DESC(Bufsize, "Buffer size (default 32K)");
+MODULE_PARM_DESC(Bufsize, "Buffer size (default 64K)");
 MODULE_PARM_DESC(Minor_nr, "Minor number to use (default: 255(=DYNAMIC))");
 #ifdef KERNEL_2_4
 MODULE_PARM(Bufsize, "i");
@@ -130,44 +146,69 @@ module_param(Minor_nr, int, S_IRUGO);
 #endif
 
 //-----------------------------------------------------------------------------
-int __init init_module(void) {
+__init int init_module(void) {
     int eax;
     kmi_miscinfo.minor = Minor_nr;
     if((eax = misc_register(&kmi_miscinfo)) != 0) {
         return eax;
     }
     Minor_nr = kmi_miscinfo.minor; // give it back to sysfs
+    printk(KERN_INFO "Registered at minor %d\n", Minor_nr);
     return 0;
 }
 
-void __exit cleanup_module(void) {
+__exit void cleanup_module(void) {
     misc_deregister(&kmi_miscinfo);
     return;
 }
 
 //-----------------------------------------------------------------------------
-static int kio_init(struct tty_struct *tty, struct tty_struct *ctl) {
-    struct rpld_packet p;
+static int kio_init(struct tty_struct *tty, struct tty_struct *ctl,
+ struct file *filp)
+{
+    /* Called from drivers/char/tty_io.c:init_dev() when the refcount of a tty
+    raises from zero to one. Usually, an EVT_OPEN follows an EVT_INIT. */
+    struct rpldev_packet p;
+    char dev[MAXFNLEN], *full_dev;
+    int len;
 
-    if(PTY_MASTER(tty)) { return 0; }
+    SKIP_PTM(tty);
+    if(IS_ERR(full_dev = d_path(filp->f_dentry, filp->f_vfsmnt,
+     dev, sizeof(dev)))) {
+        p.size = 0;
+        return mv_buffer(&p, NULL, 0);
+    }
 
+    p.size  = cpu_to_le16(len = strlen(full_dev));
     p.dev   = TTY_DEVNR(tty);
-    p.size  = 0;
     p.event = EVT_INIT;
     p.magic = MAGIC_SIG;
-    return mv_buffer(&p, NULL, 0);
+    fill_time(&p.time);
+    return mv_buffer(&p, full_dev, len);
 }
 
-static int kio_open(struct tty_struct *tty, struct tty_struct *ctl) {
-    struct rpld_packet p;
+static int kio_open(struct tty_struct *tty, struct tty_struct *ctl,
+ struct file *filp)
+{
+    /* Called from drivers/char/tty_io.c:tty_open() whenever an open() on a
+    tty succeeds. */
+    struct rpldev_packet p;
+    char dev[MAXFNLEN], *full_dev;
+    int len;
 
-    if(PTY_MASTER(tty)) { return 0; }
+    SKIP_PTM(tty);
+    if(IS_ERR(full_dev = d_path(filp->f_dentry, filp->f_vfsmnt,
+     dev, sizeof(dev)))) {
+        p.size = 0;
+        return mv_buffer(&p, NULL, 0);
+    }
 
+    p.size  = cpu_to_le16(len = strlen(full_dev));
     p.dev   = TTY_DEVNR(tty);
-    p.size  = 0;
     p.event = EVT_OPEN;
     p.magic = MAGIC_SIG;
-    return mv_buffer(&p, NULL, 0);
+    fill_time(&p.time);
+    return mv_buffer(&p, full_dev, len);
 }
 
 static int kio_read(const char __user *buf, size_t count,
@@ -175,70 +216,94 @@ static int kio_read(const char __user *buf, size_t count,
 {
     /* The data flow is a bit weird at first. kio_read() gets the data on its
     way between ttyDriver(master) -> /dev/stdin(slave), meaning this function
-    is called when you hit the keyboard. */
-    struct rpld_packet p;
+    is called when you hit the keyboard. _Even_ if you do not see any text
+    onscreen. */
+    struct rpldev_packet p;
 
-    if(count == 0 || PTY_MASTER(tty)) { return 0; }
+    SKIP_PTM(tty);
+    if(count == 0) { return 0; }
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = cpu_to_le16(count);
     p.event = EVT_READ;
     p.magic = MAGIC_SIG;
+    fill_time(&p.time);
     return mv_buffer(&p, buf, count);
 }
 
 static int kio_write(const char __user *buf, size_t count,
  struct tty_struct *tty)
 {
-    // Data flow: /dev/stdout(slave) -> tty driver(master)
-    struct rpld_packet p;
+    /* Data flow: /dev/stdout(slave) -> tty driver(master).
+    There are two ways an application can use a tty:
 
-    if(count == 0 || PTY_MASTER(tty)) { return 0; }
+    1. noecho mode (the case with interactive shells)
+       Shells print chars as they see fit, thus generating EVT_READ for you
+       hitting a key, and EVT_WRITE for the shell displaying it.
+
+    2. echo mode (cat waiting for EOF on stdin)
+       The _tty driver_ echoes chars back. This generates EVT_READ for your
+       keyboard interaction, plus one EVT_WRITE when an end-of-line is
+       received.
+
+    Sounds familiar? Anyway, don't care, we log it all in userspace. You can
+    use the "E" key during ttyreplay which shows EVT_READ events which is all
+    you need if the EVT_WRITE packets do not suffice. */
+    struct rpldev_packet p;
+
+    SKIP_PTM(tty);
+    if(count == 0) { return 0; }
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = cpu_to_le16(count);
     p.event = EVT_WRITE;
     p.magic = MAGIC_SIG;
+    fill_time(&p.time);
     return mv_buffer(&p, buf, count);
 }
 
+/*
 static int kio_ioctl(struct tty_struct *tty, struct tty_struct *ctl,
  unsigned int cmd, unsigned long arg)
 {
     struct rpld_packet p;
     uint32_t cmd32;
 
-    if(PTY_MASTER(tty)) { return 0; }
+    SKIP_PTM(tty);
 
     cmd32   = cmd;
     p.dev   = TTY_DEVNR(tty);
     p.size  = cpu_to_le16(sizeof(cmd32));
     p.event = EVT_IOCTL;
     p.magic = MAGIC_SIG;
+    fill_time(&p.time);
     return mv_buffer(&p, &cmd32, sizeof(cmd32));
 }
+*/
 
 static int kio_close(struct tty_struct *tty, struct tty_struct *other) {
-    struct rpld_packet p;
+    struct rpldev_packet p;
 
-    if(PTY_MASTER(tty)) { return 0; }
+    SKIP_PTM(tty);
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = 0;
     p.event = EVT_CLOSE;
     p.magic = MAGIC_SIG;
+    fill_time(&p.time);
     return mv_buffer(&p, NULL, 0);
 }
 
 static int kio_deinit(struct tty_struct *tty, struct tty_struct *other) {
-    struct rpld_packet p;
+    struct rpldev_packet p;
 
-    if(PTY_MASTER(tty)) { tty = other; }
+    if(IS_PTY_MASTER(tty)) { tty = other; }
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = 0;
     p.event = EVT_DEINIT;
     p.magic = MAGIC_SIG;
+    fill_time(&p.time);
     return mv_buffer(&p, NULL, 0);
 }
 
@@ -250,11 +315,8 @@ static int uio_open(struct inode *inode, struct file *filp) {
         inode->i_mode &= ~(S_IWUGO | S_IXUGO);
     }
 
-    /* The RPL device can only be opened once, since otherwise, different
-    packets could go to different readers. It is already "dangerous" with
-    fork() which can kinda step over this. Yet safe against spurious
-    open()s. */
-
+    /* The RPL device should only be opened once, since otherwise, different
+    packets could go to different readers. */
     down(&Open_lock);
     if(Open_count) {
         up(&Open_lock);
@@ -278,8 +340,8 @@ static int uio_open(struct inode *inode, struct file *filp) {
     rpl_qopen   = kio_open;
     rpl_qread   = kio_read;
     rpl_qwrite  = kio_write;
+    //rpl_qioctl  = kio_ioctl;
     rpl_qclose  = kio_close;
-    rpl_qioctl  = kio_ioctl;
     rpl_qdeinit = kio_deinit;
     up(&Buffer_lock);
 
@@ -310,7 +372,7 @@ static ssize_t uio_read(struct file *filp, char __user *buf,
     }
 
     // Data is available, so give it to the user
-    count = imin_uint(count, avail_R());
+    count = min_uint(count, avail_R());
     mv_to_user(buf, count);
 
     up(&Buffer_lock);
@@ -318,13 +380,13 @@ static ssize_t uio_read(struct file *filp, char __user *buf,
     return count;
 }
 
-static loff_t uio_llseek(struct file *filp, loff_t offset, int origin) {
+static loff_t uio_seek(struct file *filp, loff_t offset, int origin) {
     /* Since you can not seek in the circular buffer, the only purpose of
-    lseek() is to skip some bytes and "manually" advance the read pointer RP
-    (BufRP). Thus, the only accepted origin is SEEK_CUR (1), or SEEK_END (2)
-    with offset==0. */
+    seek() is to skip some bytes and "manually" advance the read pointer RP
+    (BufRP). Thus, the only accepted origin is SEEK_CUR, or SEEK_END
+    with offset == 0. */
 
-    if(origin == 2 && offset == 0) {
+    if(origin == SEEK_END && offset == 0) {
         if(down_interruptible(&Buffer_lock)) { return -ERESTARTSYS; }
         BufRP = BufWP;
         up(&Buffer_lock);
@@ -332,12 +394,11 @@ static loff_t uio_llseek(struct file *filp, loff_t offset, int origin) {
         return 0;
     }
 
-    if(origin != 1) { return -ESPIPE; }
+    if(origin != SEEK_CUR) { return -ESPIPE; }
     if(offset == 0) { return 0; }
 
     if(down_interruptible(&Buffer_lock)) { return -ERESTARTSYS; }
-    BufRP = Buffer + (BufRP - Buffer + imin_uint(offset, avail_R())) % Bufsize;
-    WARN_ON(BufRP >= Buffer + Bufsize);
+    BufRP = Buffer + (BufRP - Buffer + min_uint(offset, avail_R())) % Bufsize;
     if(BufRP >= Buffer + Bufsize) { 
         WARN_ON(1);
         BufRP = BufWP = 0;
@@ -351,8 +412,7 @@ static loff_t uio_llseek(struct file *filp, loff_t offset, int origin) {
 static int uio_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
  unsigned long arg)
 {
-    if(_IOC_TYPE(cmd) != RPL_IOC_MAGIC) { return -ENOTTY; }
-
+    if(_IOC_TYPE(cmd) != RPL_IOC_MAGIC) { return -ENOTTY; /* ENAUGHTY */ }
     switch(cmd) {
         case RPL_IOC_GETBUFSIZE:
             return Bufsize;
@@ -360,10 +420,14 @@ static int uio_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
             return avail_R();
         case RPL_IOC_GETWAVAIL:
             return avail_W();
-        default:
-            return -ENOTTY;
+        case RPL_IOC_IDENTIFY:
+            return 0xC0FFEE;
+        case RPL_IOC_SEEK:
+            return uio_seek(filp, arg, SEEK_CUR);
+        case RPL_IOC_FLUSH:
+            return uio_seek(filp, 0, SEEK_END);
     }
-    return 0;
+    return -ENOTTY;
 }
 
 static unsigned int uio_poll(struct file *filp, poll_table *wait) {
@@ -400,25 +464,48 @@ inline static ssize_t avail_W(void) {
     return BufRP - BufWP - 1;
 }
 
-inline static unsigned int imin_uint(unsigned int a, unsigned int b) {
+inline static void fill_time(struct timeval *tv) {
+    do_gettimeofday(tv);
+
+    /* The following code all gets optimized away on LE, yay! And even on BE,
+    it is reduced by 50%. Just think of how ugly a #if-#endif wrappage
+    would be. :-) */
+    if(sizeof(tv->tv_sec) == sizeof(uint32_t)) {
+        tv->tv_sec = cpu_to_le32(tv->tv_sec);
+    } else if(sizeof(tv->tv_sec) == sizeof(uint64_t)) {
+        tv->tv_sec = cpu_to_le64(tv->tv_sec);
+    }
+
+    if(sizeof(tv->tv_usec) == sizeof(uint32_t)) {
+        tv->tv_usec = cpu_to_le32(tv->tv_usec);
+    } else if(sizeof(tv->tv_usec) == sizeof(uint64_t)) {
+        tv->tv_usec = cpu_to_le64(tv->tv_usec);
+    }
+    return;
+}
+
+inline static unsigned int min_uint(unsigned int a, unsigned int b) {
     return a < b ? a : b;
 }
 
 inline static uint32_t mkdev_26(unsigned long maj, unsigned long min) {
+    /* Enforce the in-2.6-kernel scheme on userspace. That's because computing
+    the compatibility scheme (minor:12,major:12,minor:8) takes longer to
+    compute. See the optimization statement at the top. */
     return (maj << 20) | (min & 0xFFFFF);
 }
 
-static int mv_buffer(struct rpld_packet *p, const void *buf, size_t count) {
+static int mv_buffer(struct rpldev_packet *p, const void *buf, size_t count) {
     /* Copy the contents the tty driver received to our circulary buffer and
     also add a header (rpld_packet) to it so that the userspace daemon can
     recognize the type. */
     if(down_interruptible(&Buffer_lock)) { return -ERESTARTSYS; }
-    if(avail_W() < sizeof(struct rpld_packet) + count) {
+    if(avail_W() < sizeof(struct rpldev_packet) + count) {
         up(&Buffer_lock);
         return -ENOSPC;
     }
 
-    mv_buffer2(p, sizeof(struct rpld_packet));
+    mv_buffer2(p, sizeof(struct rpldev_packet));
     if(count > 0) { mv_buffer2(buf, count); }
 
     up(&Buffer_lock);
