@@ -21,6 +21,7 @@ user/global.c - Shared functions for user-space apps
 
   -- For details see the COPYING file
 =============================================================================*/
+#include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <errno.h>
@@ -30,7 +31,9 @@ user/global.c - Shared functions for user-space apps
 #include <string.h>
 #include <unistd.h>
 #include <libHX.h>
-
+#if defined(__FreeBSD__) || defined(__OpenBSD__)
+#    include "../include/rpl_ioctl.h"
+#endif
 #include "dev.h"
 #include "global.h"
 
@@ -56,6 +59,7 @@ void load_locale(const char *exe) {
         hmc_free(base);
         hmc_free(de);
     }
+
     if(!ok) {
         if(stat("locale/de/LC_MESSAGES/ttyrpld.mo", &sb) == 0) {
             bindtextdomain("ttyrpld", ".");
@@ -97,14 +101,27 @@ off_t G_skip(int fd, off_t offset, int do_wait) {
     size_t rem = offset;
     char buf[4096];
 
+#if (defined(__FreeBSD__) || defined(__OpenBSD__))
+    size_t eax = 0;
+    /* Personally, I dislike coffee. Biological disadvantages result from
+    drinking it. But it is nevertheless a working signature here. */
+//    if(D_IOCTL(fd, RPL_IOC_IDENTIFY, eax) && eax == 0xC0FFEE)
+    ioctl(fd, RPL_IOC_IDENTIFY, &eax);
+    if(eax == 0xC0FFEE) {
+        /* BSD does not have lseek() for device files.
+        And its return value scheme is also limited. */
+        eax = offset;
+        return ioctl(fd, RPL_IOC_SEEK, &eax);
+    }
+#endif
     if(seekable != -1) { return lseek(fd, offset, SEEK_CUR); }
 
     if(do_wait) {
         while(rem > 0) {
-            ssize_t eax = read(fd, buf, min_uint(sizeof(buf), rem));
-            if(eax < 0) { return -1; }
-            if(eax == rem) { return 0; }
-            rem -= eax;
+            ssize_t ebx = read(fd, buf, min_uint(sizeof(buf), rem));
+            if(ebx < 0) { return -1; }
+            if(ebx == rem) { return 0; }
+            rem -= ebx;
             sched_yield();
         }
         return 0;
@@ -112,10 +129,10 @@ off_t G_skip(int fd, off_t offset, int do_wait) {
 
     // non wait method
     while(rem > 0) {
-        ssize_t eax = read(fd, buf, min_uint(sizeof(buf), rem));
-        if(eax <= 0) { return -1; }
-        if(eax == rem) { return 0; }
-        rem -= eax;
+        ssize_t ebx = read(fd, buf, min_uint(sizeof(buf), rem));
+        if(ebx <= 0) { return -1; }
+        if(ebx == rem) { return 0; }
+        rem -= ebx;
         sched_yield();
     }
 
