@@ -21,21 +21,21 @@ user/infod.c - INFO socket for RPLD
 
   -- For details see the COPYING file
 =============================================================================*/
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
-#include <sys/types.h>
 #include <sys/un.h>
 #include <pthread.h>
 #include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
-#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
 #include <unistd.h>
 
 #include <libHX.h>
+#include "rpl_stdint.h"
 #include "dev.h"
 #include "global.h"
 #include "rdsh.h"
@@ -98,24 +98,15 @@ static void *client_thread(void *arg) {
     while(1) {
         static const uint32_t zero = 0;
         unsigned char req;
-        uint32_t gint; // general int
-        //char buf[4096];
+        uint32_t gint;
+        int eax = 0;
 
-        if(!RECEIVE_FULL(fd, &req, sizeof(unsigned char)) || req == IFP_NONE) {
-            break;
-        } else if(req > IFT_int_min && req < IFT_int_max) {
-            if(!RECEIVE_FULL(fd, &gint, sizeof(uint32_t))) { break; }
-            SWAB(&gint, sizeof(uint32_t));
-        } /* else if(req > IFT_str_min && req < IFT_str_max) {
-            if(!RECEIVE_FULL(fd, &gint, sizeof(uint32_t))) { break; }
-            SWAB(&gint, sizeof(uint32_t));
-            if(!RECEIVE_FULL(fd, buf, min_uint(sizeof(buf), gint))) { break; }
-        } */
+        eax |= !RECEIVE_FULL(fd, &req, sizeof(unsigned char));
+        eax |= !RECEIVE_FULL(fd, &gint, sizeof(gint));
+        SWAB(&gint, sizeof(gint));
+        if(eax || req == IFP_NONE) { break; }
 
         switch(req) {
-            // Voids
-
-            // Integers
             case IFP_ACTIVATE ... IFP_DEACTIVSES:
                 set_session_status(gint, req);
                 break;
@@ -135,11 +126,6 @@ static void *client_thread(void *arg) {
             case IFP_ZERO:
                 zero_counters(gint);
                 break;
-
-            // Strings
-            // currently none
-
-            // Other
             default:
                 notify(LOG_WARNING, _("[infod] Unknown request %d, closing"
                  " connection to not run into conversion problems.\n"), req);
@@ -165,8 +151,8 @@ static void getinfo_text(uint32_t dev, int fd) {
         skprintf(fd,
           "=========================================================="
           "================\n"
-          "  *       %9llu %9llu ttyrpld " TTYRPLD_VERSION "\n"
-          "  IOCD: %zu/%zu/%zu/%zu  RW: %zu/%zu  I: %zu  B: %zu\n"
+          "  *           %9llu %9llu ttyrpld " TTYRPLD_VERSION "\n"
+          "  IOCD: %lu/%lu/%lu/%lu  RW: %lu/%lu  I: %lu  B: %lu\n"
           "----------------------------------------------------------"
           "----------------\n",
           Stats.in, Stats.out, Stats.init, Stats.open, Stats.close,
@@ -202,8 +188,8 @@ static void getinfo(uint32_t dev, int fd) {
         struct HXbtree_node *nd;
         struct HXbtrav *travp;
 
-        skprintf(fd, "ttyrpld " TTYRPLD_VERSION "\n" "format 2\n"
-          "%zu %zu %zu %zu %zu %zu %llu %llu %zu %zu\n",
+        skprintf(fd, "ttyrpld " TTYRPLD_VERSION "\n" "format 3\n"
+          "%lu %lu %lu %lu %lu %lu %llu %llu %lu %lu\n",
           Stats.init, Stats.open, Stats.close, Stats.deinit, Stats.read,
           Stats.write, Stats.in, Stats.out, Stats.ioctl, Stats.badpack
         );
@@ -289,36 +275,28 @@ static void block_signals(void) {
 }
 
 static void print_entry_text(int fd, struct tty *tty) {
-    char buf[64], status;
-
+    char status = '?';
     switch(tty->status) {
-        case IFP_ACTIVATE:   status = '*'; break;
-        case IFP_DEACTIVATE: status = ' '; break;
-        case IFP_DEACTIVSES: status = '.'; break;
-        default:             status = '?'; break;
+        case IFP_ACTIVATE:   status = 'A'; break;
+        case IFP_DEACTIVATE: status = 'D'; break;
+        case IFP_DEACTIVSES: status = 'S'; break;
     }
-
-    G_devname_nm(tty->dev, buf, sizeof(buf));
-    skprintf(fd, "%c %-7s %9zu %9zu %s\n", status,
-      buf, tty->in, tty->out,
-      basename_pp(tty->file, GOpt.ofmt)
+    skprintf(fd, "%c %-11s %9lu %9lu %s\n", status,
+      tty->sdev, tty->in, tty->out,
+      basename_pp(tty->log, GOpt.ofmt)
     );
     return;
 }
 
 static void print_entry(int fd, struct tty *tty) {
-    char buf[64], status = 'x';
-
+    char status = '?';
     switch(tty->status) {
         case IFP_ACTIVATE:   status = 'A'; break;
         case IFP_DEACTIVATE: status = 'D'; break;
         case IFP_DEACTIVSES: status = 'S'; break;
-        default:             status = '?'; break;
     }
-
-    G_devname_nm(tty->dev, buf, sizeof(buf));
-    skprintf(fd, "%c %ld %ld %s %zu %zu %s\n",
-     status, KD26_PARTS(tty->dev), buf, tty->in, tty->out, tty->file);
+    skprintf(fd, "%c %ld %ld %s %lu %lu %s\n",
+     status, KD26_PARTS(tty->dev), tty->sdev, tty->in, tty->out, tty->log);
     return;
 }
 
