@@ -70,7 +70,7 @@ struct {
 
 // Functions
 static int replay_file(int, const char *);
-static void e_proc(int, struct disk_packet *, struct pctrl_info *, char *,
+static void e_proc(int, struct rpldsk_packet *, struct pctrl_info *, char *,
   struct timeval *, unsigned long *);
 
 static unsigned long calc_ovcorr(unsigned long, int);
@@ -151,7 +151,7 @@ int main(int argc, const char **argv) {
 }
 
 static int replay_file(int fd, const char *name) {
-    struct disk_packet packet;
+    struct rpldsk_packet packet;
     struct pctrl_info ps;
     struct timeval stamp;
     ssize_t eax;
@@ -167,8 +167,8 @@ static int replay_file(int fd, const char *name) {
     ps.skval  = Opt.skval;
     pctrl_activate(&ps);
 
-    while((eax = read_waitfm(fd, &packet, sizeof(struct disk_packet))) ==
-     sizeof(struct disk_packet)) {
+    while((eax = read_waitfm(fd, &packet, sizeof(struct rpldsk_packet))) ==
+     sizeof(struct rpldsk_packet)) {
         if(packet.magic != MAGIC_SIG) {
             fprintf(stderr, _("\n" "<Packet inconsistency! "
              "Trying to find next valid packet.>\n"));
@@ -187,20 +187,25 @@ static int replay_file(int fd, const char *name) {
             case EVT_WRITE:
                 e_proc(fd, &packet, &ps, &tick, &stamp, &skew);
                 break;
-            case EVT_IDENT:
-                fprintf(stderr, "\n\e[1;37m%s ", _("Created using")); // ]
+            case EVT_ID_PROG:
+                fprintf(stderr, "\e[1;37;41m%s ", _("Created using")); // ]
+                read_through(fd, STDERR_FILENO, packet.size);
+                fprintf(stderr, "\e[0m\n"); // ]
+                break;
+            case EVT_ID_DEVPATH:
+                fprintf(stderr, "\e[1;37;41m%s: ", _("Device path")); // ]
                 read_through(fd, STDERR_FILENO, packet.size);
                 fprintf(stderr, "\e[0m\n"); // ]
                 break;
             case EVT_DEINIT:
-                fprintf(stderr, "\n\e[1;37;41m%s\e[0m\n", // ]]
+                fprintf(stderr, "\n\e[1;37;42m%s\e[0m\n", // ]]
                  _("<tty device has been closed>"));
                 // fallthrough kinda
                 read_nullfm(fd, packet.size);
                 break;
             case EVT_READ:
                 if(ps.echo) {
-                    printf("\e[1;37;41m"); // ]
+                    printf("\e[1;37;45m"); // ]
                     e_proc(fd, &packet, &ps, &tick, &stamp, &skew);
                     printf("\e[0m"); // ]
                     break;
@@ -227,7 +232,7 @@ static int replay_file(int fd, const char *name) {
     return ps.brk;
 }
 
-static void e_proc(int fd, struct disk_packet *p, struct pctrl_info *i,
+static void e_proc(int fd, struct rpldsk_packet *p, struct pctrl_info *i,
   char *tick, struct timeval *stamp, unsigned long *skew)
 {
     if(i->sktype == PCTRL_SKPACK && i->skval-- > 0) {
@@ -240,7 +245,7 @@ static void e_proc(int fd, struct disk_packet *p, struct pctrl_info *i,
         // No delay after the first packet has been read...
         ++*tick;
         if(i->skval > 0) {
-            memcpy(stamp, &p->tv, sizeof(struct timeval));
+            memcpy(stamp, &p->time, sizeof(struct timeval));
             read_through(fd, STDOUT_FILENO, p->size);
             return;
         }
@@ -250,11 +255,11 @@ static void e_proc(int fd, struct disk_packet *p, struct pctrl_info *i,
         FM_LIVE mode, because we have been already waiting enough with
         read_wait(). */
         struct timeval delta;
-        tv_delta(stamp, &p->tv, &delta);
+        tv_delta(stamp, &p->time, &delta);
 
         if(i->sktype == PCTRL_SKTIME &&
          (i->skval -= tv2usec(&delta) / 1000) > 0) {
-            memcpy(stamp, &p->tv, sizeof(struct timeval));
+            memcpy(stamp, &p->time, sizeof(struct timeval));
             read_through(fd, STDOUT_FILENO, p->size);
             return;
         }
@@ -271,13 +276,13 @@ static void e_proc(int fd, struct disk_packet *p, struct pctrl_info *i,
 
     while(i->paused) { sched_yield(); }
 
-    memcpy(stamp, &p->tv, sizeof(struct timeval));
+    memcpy(stamp, &p->time, sizeof(struct timeval));
     read_through(fd, STDOUT_FILENO, p->size);
 
     if(Opt.showtime) {
         struct tm tm;
         char buf[80];
-        time_t now = p->tv.tv_sec;
+        time_t now = p->time.tv_sec;
         localtime_r(&now, &tm);
         if(Opt.showtime == 1) {
             strftime(buf, sizeof(buf), "\e7\e[1;300H\e[9D" // ]]
@@ -316,11 +321,11 @@ static unsigned long calc_ovcorr(unsigned long ad, int rd) {
 static int find_next_packet(int fd) {
     /* There is more algorithmic code than simple code in ttyreplay.
     Outstanding. */
-#define LZ           sizeof(struct disk_packet)
+#define LZ           sizeof(struct rpldsk_packet)
 #define BZ           (2 * LZ)
-#define MAGIC_OFFSET offsetof(struct disk_packet, magic)
+#define MAGIC_OFFSET offsetof(struct rpldsk_packet, magic)
     char buf[BZ];
-    struct disk_packet *packet = (void *)buf;
+    struct rpldsk_packet *packet = (void *)buf;
     size_t s;
     int ok = 0;
 
@@ -336,7 +341,7 @@ static int find_next_packet(int fd) {
         if((ptr = memchr(buf, MAGIC_SIG, BZ)) != NULL &&
          ptr - buf >= MAGIC_OFFSET) {
             // A magic byte has been found and the packet start is complete
-            char *ctx = containerof(ptr, struct disk_packet, magic);
+            char *ctx = containerof(ptr, struct rpldsk_packet, magic);
             if(ctx != buf) {
                 size_t cnt = buf + BZ - ctx;
                 ctx = memmove(buf, ctx, cnt);
@@ -551,7 +556,8 @@ static ssize_t read_waitfm(int fd, void *buf, size_t count) {
     the file size increases inbetween. */
     if(Opt.follow == FM_CATCHUP && (pos = lseek(fd, 0, SEEK_CUR)) != 0 &&
      fstat(fd, &sb) == 0 && pos == sb.st_size) {
-        fprintf(stderr, _("\n" "<Switching to live feed follow mode>\n"));
+        fprintf(stderr, "\n\e[1;37;41m%s\e[0m\n",
+         _("<Switching to live feed follow mode>"));
         Opt.follow = FM_LIVE;
     }
 
