@@ -21,10 +21,10 @@ user/rplctl.c - INFOD client
 
   -- For details see the COPYING file
 =============================================================================*/
+#include <sys/types.h>
 #include <sys/select.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
-#include <sys/types.h>
 #include <sys/un.h>
 #include <errno.h>
 #include <popt.h>
@@ -43,8 +43,6 @@ user/rplctl.c - INFOD client
 // Function prototypes
 static uint32_t getdev(const char *);
 static void send_int(int, unsigned char, uint32_t);
-//static void send_void(int, unsigned char);
-//static void send_string(int, unsigned char, const char *);
 static int unix_client(const char *);
 
 static int get_options(int *, const char ***);
@@ -68,7 +66,7 @@ int rplctl_main(int argc, const char **argv) {
 
     if(get_options(&argc, &argv) == 0) {
         if(!Opt.parseable) {
-            printf("A %-3s %13s %9s %s\n", _("TTY"), _("BYTES IN"),
+            printf("A %-7s %13s %9s %s\n", _("TTY"), _("BYTES IN"),
              _("OUT"), _("FILENAME"));
         }
         send_int(Sockfd, IFP_GETINFO_A, 0);
@@ -166,25 +164,6 @@ static void send_int(int fd, unsigned char req, uint32_t dev) {
     return;
 }
 
-/*
-static void send_void(int fd, unsigned char req) {
-    send_wait(fd, &req, sizeof(unsigned char), 0);
-    read_reply(fd);
-    return;
-}
-
-static void send_string(int fd, unsigned char req, const char *str) {
-    size_t size = strlen(str);
-    uint32_t szpack = size;
-    send_wait(fd, &req, sizeof(unsigned char), 0);
-    SWAB(&szpack, sizeof(uint32_t));
-    send_wait(fd, &szpack, sizeof(uint32_t), 0);
-    send_wait(fd, str, size, 0);
-    read_reply(fd);
-    return;
-}
-*/
-
 static int unix_client(const char *port) {
     struct sockaddr_un sk;
     int fd = -1;
