@@ -21,10 +21,10 @@ user/rpld.c - User space daemon (filtering, etc.)
 
   -- For details see the COPYING file
 =============================================================================*/
+#include <sys/types.h>
 #include <sys/mman.h> // mlock()
 #include <sys/stat.h>
 #include <sys/time.h> // gettimeofday()
-#include <sys/types.h>
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -32,7 +32,7 @@ user/rpld.c - User space daemon (filtering, etc.)
 #include <pthread.h>
 #include <pwd.h>
 #include <signal.h>
-#include <stdint.h>
+#include "rpl_stdint.h"
 #include <stdio.h>
 #include <stdlib.h>
 extern int posix_memalign(void **, size_t, size_t);
@@ -43,27 +43,38 @@ extern int posix_memalign(void **, size_t, size_t);
 
 #include <libHX.h>
 #include "dev.h"
+#include "rpl_endian.h"
 #include "rpl_ioctl.h"
 #include "rpl_packet.h"
 #include "global.h"
 #include "rdsh.h"
-#include "stat2.h"
-#define NODEPATH "/var/run/.rpldev"
+
+#ifndef S_IRUGO
+#    define S_IRUGO (S_IRUSR | S_IRGRP | S_IROTH)
+#endif
+#ifndef S_IWUGO
+#    define S_IWUGO (S_IWUSR | S_IWGRP | S_IWOTH)
+#endif
+#ifndef S_IXUGO
+#    define S_IXUGO (S_IXUSR | S_IXGRP | S_IXOTH)
+#endif
 
 // The prototype order is the same as the function body order
 static void mainloop(int);
 
-static int packet_preprox(struct mem_packet *);
-static int packet_process(struct mem_packet *, struct tty *, int);
+static int packet_preprox(struct rpldev_packet *);
+static int packet_process(struct rpldev_packet *, struct tty *, int);
 
-static void log_cuid(struct tty *);
-static int log_open(struct tty *);
-static void log_write(struct mem_packet *, struct tty *, int);
+static int evt_open(struct rpldev_packet *, struct tty *, int);
+static void log_open(struct tty *);
+static void log_write(struct rpldev_packet *, struct tty *, int);
+
+static int check_parent_directory(const char *);
+static void fill_info(struct tty *, const char *);
 
 static int init_device(const char *);
 static int init_sbuffer(int);
 static void deinit_sbuffer(void);
-static const char *mknod_rpldev(void);
 static int init_sighandler(void);
 static void sighandler_int(int);
 static void sighandler_alrm(int);
@@ -75,6 +86,8 @@ static int read_config_bp(const char *, const char *);
 static int pconfig_user(const char *, unsigned char, void *, void *);
 
 inline static const char *basename(const char *);
+static int find_devnode(uint32_t, char *, size_t, const char **);
+static int find_devnode_dive(uint32_t, char *, size_t, const char *);
 static char *getnamefromuid(uid_t, char *, size_t);
 static uid_t getuidfromname(const char *);
 
@@ -82,17 +95,16 @@ static uid_t getuidfromname(const char *);
 static void *Buffer = NULL;
 static struct {
     int _running, dolog;
-    char *ovdev, *fbdev;
+    char *device;
     long bsize;
 
     int infod_start;
 } Opt = {
     ._running    = 1,
-    .bsize       = 16384,
-    .ovdev       = NULL, // -D
-    .fbdev       = "/dev/rpl:/dev/misc/rpl",
-    .dolog       = 1,    // !-Q
-    .infod_start = 0,    // -I
+    .dolog       = 1, // !-Q
+    .bsize       = 32 * 1024,
+    .device      = "/dev/misc/rpl:/dev/rpl", // -D
+    .infod_start = 0, // -I
 };
 
 //-----------------------------------------------------------------------------
@@ -135,7 +147,7 @@ int main(int argc, const char **argv) {
         return EXIT_FAILURE;
     }
 
-    if((fd = init_device((Opt.ovdev != NULL) ? Opt.ovdev : Opt.fbdev)) < 0) {
+    if((fd = init_device(Opt.device)) < 0) {
         fprintf(stderr, _("No device could be opened, aborting.\n"));
         return EXIT_FAILURE;
     }
@@ -143,14 +155,11 @@ int main(int argc, const char **argv) {
     if(init_sbuffer(fd) < 0) { return EXIT_FAILURE; }
     init_sighandler();
 
-    // Callback tick for statistics
-    if(GOpt.verbose)    { alarm(1); }
-    if(Opt.infod_start) { infod_init(); }
-    if(GOpt.syslog)     { openlog("rpld", LOG_PID, LOG_DAEMON); }
-
-    // Not seteuid(), because it shall not be posibble to regain privs.
+    if(Opt.infod_start)  { infod_init(); }
+    if(GOpt.syslog)      { openlog("rpld", LOG_PID, LOG_DAEMON); }
     if(GOpt.user_id > 0) { setuid(GOpt.user_id); }
-    if(Opt.infod_start) { pthread_create(&infod_id, NULL, infod_main, NULL); }
+    if(Opt.infod_start)  { pthread_create(&infod_id, NULL, infod_main, NULL); }
+    if(GOpt.verbose)     { alarm(1); }
     mainloop(fd);
 
     if(Opt.infod_start) {
@@ -166,17 +175,29 @@ int main(int argc, const char **argv) {
 
 static void mainloop(int fd) {
     while(Opt._running) {
-        struct mem_packet packet;
+        struct rpldev_packet packet;
         struct tty *tty;
+        ssize_t eax;
 
-        if(read(fd, &packet, sizeof(struct rpld_packet)) <
-         (ssize_t)sizeof(struct rpld_packet)) {
+        if((eax = read(fd, &packet, sizeof(struct rpldev_packet))) <
+         (ssize_t)sizeof(struct rpldev_packet)) {
+            struct stat sb;
+            fstat(fd, &sb);
+            if(!S_ISREG(sb.st_mode)) {
+#ifdef __OpenBSD__
+                if(errno == EINTR) { continue; }
+#endif
+                fprintf(stderr, _("\n" "Short read: %ld bytes only. Error %d:"
+                 " %s\n"), (long)eax, errno, strerror(errno));
+            }
             Opt._running = 0;
             break;
         }
 
         SWAB(&packet.dev, sizeof(packet.dev));
         SWAB(&packet.size, sizeof(packet.size));
+        SWAB(&packet.time.tv_sec, sizeof(packet.time.tv_sec));
+        SWAB(&packet.time.tv_usec, sizeof(packet.time.tv_usec));
 
         if(packet.magic != MAGIC_SIG) {
             ++Stats.badpack;
@@ -185,10 +206,6 @@ static void mainloop(int fd) {
             continue;
         }
 
-        /* Timestamp is taken here instead of in the Kernel module to reduce
-        the amount of data which has to be copied over the device. */
-        gettimeofday(&packet.tv, NULL);
-
         if(!packet_preprox(&packet)) {
             G_skip(fd, packet.size, 0);
             continue;
@@ -202,8 +219,8 @@ static void mainloop(int fd) {
         }
 
         if(!packet_process(&packet, tty, fd)) {
-            /* packet_process is always a success, but it returns 0 to indicate
-            if it wnats to skip the payload. */
+            /* packet_process always succeeds, but it returns 0 to indicate if
+            it wants to skip the payload. */
             G_skip(fd, packet.size, 0);
         }
         pthread_mutex_unlock(&Ttys_lock);
@@ -213,8 +230,8 @@ static void mainloop(int fd) {
 }
 
 //-----------------------------------------------------------------------------
-static int packet_preprox(struct mem_packet *packet) {
-    static int *tab[] = {
+static int packet_preprox(struct rpldev_packet *packet) {
+    static unsigned long *tab[] = {
         [EVT_INIT]   = &Stats.init,
         [EVT_OPEN]   = &Stats.open,
         [EVT_READ]   = &Stats.read,
@@ -258,15 +275,29 @@ static int packet_preprox(struct mem_packet *packet) {
     return 1;
 }
 
-static int packet_process(struct mem_packet *packet, struct tty *tty, int fd) {
-    if(tty->status != IFP_ACTIVATE) { return 0; }
+static int packet_process(struct rpldev_packet *packet, struct tty *tty,
+ int fd)
+{
+    if(tty->status == IFP_DEFAULT) {
+        fill_info(tty, NULL);
+        tty->status = Opt.dolog ? IFP_ACTIVATE : IFP_DEACTIVATE;
+    }
+
+    if(tty->status != IFP_ACTIVATE) {
+        switch(packet->event) {
+            case EVT_READ:
+                tty->in += packet->size;
+                break;
+            case EVT_WRITE:
+                tty->out += packet->size;
+                break;
+        }
+        return 0;
+    }
 
     switch(packet->event) {
         case EVT_OPEN:
-            /* EVT_OPEN will not handle logfile opening since log_write() will
-            do that when a EVT_{READ,WRITE} pops up. */
-            log_cuid(tty);
-            break;
+            return evt_open(packet, tty, fd);
         case EVT_READ:
             tty->in += packet->size;
             log_write(packet, tty, fd);
@@ -288,19 +319,85 @@ static int packet_process(struct mem_packet *packet, struct tty *tty, int fd) {
 }
 
 //-----------------------------------------------------------------------------
-static void log_cuid(struct tty *tty) {
+static int evt_open(struct rpldev_packet *packet, struct tty *tty, int fd) {
+    /* OPEN event:
+    - Read the dentry from rpldev to save looking up the device node
+    - Fill basic variables (.sdev and .log)
+    - Do NOT open the logfile
+    - Find out about UID changes on the device and start a new logfile if owner
+      has changed.
+    */
     struct stat sb;
-    char buf[32];
+    int rv = 0;
 
-    if(tty->uid == -1) { return; }
-    G_devname_fs(tty->dev, buf, sizeof(buf));
-    if(stat(buf, &sb) == 0 && sb.st_uid != tty->uid) {
+    if(tty->sdev == NULL) {
+        char *buf = alloca(packet->size + 1);
+        read(fd, buf, packet->size);
+        buf[packet->size] = '\0';
+        rv = 1;
+        fill_info(tty, buf);
+    }
+
+    if(tty->uid != -1 && tty->full_dev != NULL &&
+     stat(tty->full_dev, &sb) == 0 && sb.st_uid != tty->uid) {
+        // Create new logfile
         tty->in = tty->out = 0;
         log_open(tty);
     }
+
+    return rv;
+}
+
+static void log_open(struct tty *tty) {
+    struct rpldsk_packet p = {
+        .event = EVT_ID_PROG,
+        .magic = MAGIC_SIG,
+        .time  = {-1, -1},
+    };
+    char buf[MAXFNLEN];
+    size_t s;
+
+    if(check_parent_directory(tty->log) <= 0) {
+        notify(LOG_ERR, _("Directory permission denied: It won't be possible"
+         " to write to the file %s, expect warnings.\n"), buf);
+    }
+
+    tty->fd = open(tty->log, O_WRONLY | O_CREAT | O_APPEND, S_IWUSR);
+
+    /* Add an optional ident header to record the version of which this logfile
+    was created with. Also, add the full path name of the tty for reference. */
+    strncpy(buf, "ttyrpld " TTYRPLD_VERSION, sizeof(buf) - 1);
+    buf[sizeof(buf) - 1] = '\0';
+    s = p.size = strlen(buf);
+    SWAB(&p.size, sizeof(p.size));
+    write(tty->fd, &p, sizeof(struct rpldsk_packet));
+    write(tty->fd, buf, s);
+
+    p.event = EVT_ID_DEVPATH;
+    s = p.size = strlen(tty->full_dev);
+    SWAB(&p.size, sizeof(p.size));
+    write(tty->fd, &p, sizeof(struct rpldsk_packet));
+    write(tty->fd, tty->full_dev, s);
     return;
 }
 
+static void log_write(struct rpldev_packet *packet, struct tty *tty, int fd) {
+    char *buffer = alloca(packet->size);
+    int have;
+
+    if(tty->fd < 0) { log_open(tty); }
+    if((have = read(fd, buffer, packet->size)) <= 0) { return; }
+    if(have != packet->size) { packet->size = have; }
+
+    SWAB(&packet->size, sizeof(packet->size));
+    SWAB(&packet->time.tv_sec, sizeof(packet->time.tv_sec));
+    SWAB(&packet->time.tv_usec, sizeof(packet->time.tv_usec));
+    write(tty->fd, &packet->size, sizeof(struct rpldsk_packet));
+    write(tty->fd, buffer, have);
+    return;
+}
+
+//-----------------------------------------------------------------------------
 static int check_parent_directory(const char *s) {
     char *path = alloca(strlen(s) + 1), *p;
     strcpy(path, s);
@@ -312,25 +409,82 @@ static int check_parent_directory(const char *s) {
     return HX_mkdir(path); // like `mkdir -p`
 }
 
-static int log_open(struct tty *tty) {
-    char buf[MAXFNLEN], devnode[32], fmday[16], fmtime[16], user[32];
-    struct HX_repmap catalog[] = {{'d', NULL, fmday}, {'l', NULL, devnode},
-     {'t', NULL, fmtime}, {'u', NULL, user}, {0}};
+static void fill_info(struct tty *tty, const char *aux_sdev) {
+    char full_dev[MAXFNLEN], sdev[MAXFNLEN], buf[MAXFNLEN],
+     fmday[16], fmtime[16], user[32];
+    struct HX_repmap catalog[] = {
+        {'d', NULL, fmday},
+        {'l', NULL, sdev},
+        {'t', NULL, fmtime},
+        {'u', NULL, user},
+        {0}
+    };
+    const char *pbase = NULL;
+    struct stat sb;
+    int i = 0;
 
-    { // User
-      struct stat sb;
-      G_devname_fs(tty->dev, devnode, sizeof(devnode));
-      if(stat(devnode, &sb) != 0) {
-          strcpy(user, _("NONE"));
-      } else {
-          tty->uid = sb.st_uid;
-          if(getnamefromuid(sb.st_uid, user, sizeof(user)) == NULL) {
-              snprintf(user, sizeof(user), "%d", sb.st_uid);
-          }
-      }
+    /* The rpldev kernel module provides us with the real dentry name
+    (aux_sdev) that was used open the device. Use it, if available. */
+    if(aux_sdev != NULL && *aux_sdev != '\0') {
+        const char **dirp = Device_dirs;
+        strncpy(full_dev, aux_sdev, MAXFNLEN);
+        full_dev[MAXFNLEN - 1] = '\0';
+        while(*dirp != NULL) {
+            if(strncmp(full_dev, *dirp, strlen(*dirp)) == 0) {
+                pbase = *dirp;
+                break;
+            }
+            ++dirp;
+        }
+        if(pbase == NULL && *full_dev == '/') { pbase = ""; }
+    } else if(!find_devnode(tty->dev, full_dev, sizeof(full_dev), &pbase)) {
+        /* Tricky code. find_devnode() is always called [if aux_sdev == NULL],
+        and this branch containing the sprintf is only executed if no devnode
+        was found! In that case, we use [MAJOR:MINOR] as a fictitious filename.
+        Of course, we will not be able to get the UID then. */
+        snprintf(full_dev, sizeof(full_dev),
+         "[%lu:%lu]", KD26_PARTS(tty->dev));
     }
 
-    { // Time
+    /* rpld is able to sort logs by user (by putting each user's logfiles into
+    a separate directory) -- for that, we need the username. */
+    if(stat(full_dev, &sb) != 0) {
+        // This will happen if we get a [MAJOR:MINOR] name...
+        strcpy(user, _("NONE"));
+    } else {
+        tty->uid = sb.st_uid;
+        if(getnamefromuid(sb.st_uid, user, sizeof(user)) == NULL) {
+            // Well, at least a UID.
+            snprintf(user, sizeof(user), "%d", sb.st_uid);
+        }
+    }
+
+    /* The filename in sdev contains a common prefix, such as "/dev", and may
+    contain further slashes, e.g. as in "pts/2". We must exchange them since a
+    filename cannot contain a slash -- it would otherwise always be treated as
+    another directory component. */
+    if(pbase != NULL) {
+        /* +1 in source argument: do not copy 2nd slash in e.g. "/dev/tty1"
+        no -1 in length argument: _do_ copy trailing '\0'. */
+        memmove(sdev, full_dev + strlen(pbase) + 1,
+         strlen(full_dev) - strlen(pbase));
+    } else {
+        // Usually this is [MAJOR:MINOR]
+        strncpy(sdev, full_dev, sizeof(sdev) - 1);
+        sdev[sizeof(sdev) - 1] = '\0';
+    }
+    while(i < sizeof(sdev)) {
+        if(sdev[i] == '/') { sdev[i] = '-'; }
+        ++i;
+    }
+
+    // Keep the device path that was used first
+    if(tty->sdev == NULL) {
+        HX_strclone(&tty->sdev, sdev);
+        HX_strclone(&tty->full_dev, full_dev);
+    }
+
+    { // What would be a logfile without a proper timestamp?
       time_t now = time(NULL);
       struct tm now_tm;
       localtime_r(&now, &now_tm);
@@ -338,118 +492,50 @@ static int log_open(struct tty *tty) {
       strftime(fmtime, sizeof(fmtime), "%H%M%S", &now_tm);
     }
 
-    // Simple tty name
-    G_devname_nm(tty->dev, devnode, sizeof(devnode));
-    HX_strrep(buf, MAXFNLEN, GOpt.ofmt, catalog);
-
-    // Open log
-    HX_strclone(&tty->file, buf);
-    if(!Opt.dolog) {
-        snprintf(buf, MAXFNLEN, "/dev/null");
-    } else if(check_parent_directory(buf) <= 0) {
-        notify(LOG_ERR, _("Directory permission denied: It won't be possible"
-         " to write to the file %s, expect warnings.\n"), buf);
-    }
-    if((tty->fd = open(buf, O_WRONLY | O_CREAT | O_APPEND, S_IWUSR)) < 0) {
-        notify(LOG_ERR, _("Could not open %s: %s\n"), buf, strerror(errno));
-    }
-
-    // Ident header (optional)
-    if(tty->fd >= 0) {
-        struct disk_packet p = {
-            .event = EVT_IDENT,
-            .magic = MAGIC_SIG,
-            .tv    = {-1, -1},
-        };
-        size_t s;
-
-        buf[MAXFNLEN - 1] = '\0';
-        strncpy(buf, "ttyrpld " TTYRPLD_VERSION, MAXFNLEN - 1);
-        s = p.size = strlen(buf);
-        SWAB(&p.size, sizeof(p.size));
-        write(tty->fd, &p, sizeof(struct disk_packet));
-        write(tty->fd, buf, s);
-    }
-
-    return tty->fd;
-}
-
-static void log_write(struct mem_packet *packet, struct tty *tty, int fd) {
-    /* Slap a timestamp around the packet and write it out to disk. Open the
-    log if necessary. */
-    char *buffer = alloca(packet->size);
-    int have;
-
-    if(tty->fd == -1) { log_open(tty); }
-    if((have = read(fd, buffer, packet->size)) <= 0) { return; }
-    if(have != packet->size) { packet->size = have; }
-
-    SWAB(&packet->size, sizeof(packet->size));
-    SWAB(&packet->tv.tv_sec, sizeof(packet->tv.tv_sec));
-    SWAB(&packet->tv.tv_usec, sizeof(packet->tv.tv_usec));
-    write(tty->fd, &packet->size, sizeof(struct disk_packet));
-    write(tty->fd, buffer, have);
+    HX_strrep(buf, sizeof(buf), GOpt.ofmt, catalog);
+    HX_strclone(&tty->log, buf);
     return;
 }
 
 //-----------------------------------------------------------------------------
 static int init_device(const char *in_devs) {
-    const char *new_dev = NULL;
+    char *copy = HX_strdup(in_devs), *workp = copy, *devp;
     int fd = -1;
 
-    if(Opt.ovdev == NULL && (new_dev = mknod_rpldev()) != NULL) {
-        if((fd = open(new_dev, O_RDONLY)) == -1) {
-            int se = errno;
-            new_dev = NULL;
-            perror(_("dynamic_find(): Could not open RPL device"));
-            if(se == EBUSY) {
-                fprintf(stderr, _("\t" "The RPL device can only be opened"
-                 " once,\n\t" "there is probably an instance of rpld"
-                 " running!\n"));
-            }
-        } else if(GOpt.verbose) {
-            printf(_("Successfully opened RPL device\n"));
+    while((devp = HX_strsep(&workp, ":")) != NULL) {
+        if(devp[0] == '-' && devp[1] == '\0') {
+            fd = STDIN_FILENO;
+            if(GOpt.verbose) { printf(_("Connected to %s\n"), "<stdin>"); }
+            break;
         }
-        unlink(NODEPATH);
-        unlink(new_dev);
-    }
-
-    if(new_dev == NULL) {
-        /* No, this will not change into an else if, because new_dev might be
-        set to something else above! */
-        char *copy = HX_strdup(in_devs), *devs = copy, *devp;
-
-        while((devp = HX_strsep(&devs, ":")) != NULL) {
-            if((fd = open(devp, O_RDONLY)) >= 0) {
-                if(GOpt.verbose) { printf(_("Connected to %s\n"), devp); }
-                break;
-            }
-            if(errno != ENOENT) {
-                fprintf(stderr, _("static_find(): Could not open %s even"
-                 " though it exists: %s (trying next device)\n"),
-                 devp, strerror(errno));
-            }
-            if(errno == EBUSY) {
-                fprintf(stderr, _("\t" "The RPL device can only be opened"
-                 " once,\n\t" "there is probably an instance of rpld"
-                 " running!\n"));
-            }
-            ++devp;
+        if((fd = open(devp, O_RDONLY)) >= 0) {
+            if(GOpt.verbose) { printf(_("Connected to %s\n"), devp); }
+            break;
+        }
+        if(errno != ENOENT) {
+            fprintf(stderr, _("static_find(): Could not open %s even"
+             " though it exists: %s (trying next device)\n"),
+             devp, strerror(errno));
+        }
+        if(errno == EBUSY) {
+            fprintf(stderr, _("\t" "The RPL device can only be opened"
+             " once,\n\t" "there is probably an instance of rpld"
+             " running!\n"));
         }
-        free(copy);
     }
 
+    free(copy);
     return fd;
 }
 
 static int init_sbuffer(int fd) {
-    int eax;
+    int eax = -1;
 
-    if((eax = ioctl(fd, RPL_IOC_GETBUFSIZE, 0)) > 0) {
+    if(D_IOCTL(fd, RPL_IOC_GETBUFSIZE, eax) && eax > 0) {
         Opt.bsize = eax;
     }
 
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__OpenBSD__)
     if((Buffer = malloc(Opt.bsize)) == NULL)
 #else
     if(posix_memalign(&Buffer, sysconf(_SC_PAGESIZE), Opt.bsize) != 0)
@@ -472,52 +558,6 @@ static void deinit_sbuffer(void) {
     return;
 }
 
-static const char *mknod_rpldev(void) {
-    int32_t minor = -1;
-    char buf[128];
-    FILE *fp;
-
-    if((fp = fopen("/sys/module/rpldev/Minor_nr", "r")) != NULL) {
-        fgets(buf, sizeof(buf), fp);
-        minor = strtol(buf, NULL, 0);
-    } else if((fp = fopen("/proc/misc", "r")) != NULL) {
-        while(fgets(buf, sizeof(buf), fp) != NULL) {
-            char *ptr = buf, *name;
-            while(!isdigit(*ptr)) { ++ptr; }
-            minor = strtol(ptr, NULL, 0);
-            while(isdigit(*ptr)) { ++ptr; }
-            while(isspace(*ptr)) { ++ptr; }
-
-            name = ptr;
-            while(isalnum(*ptr)) { ++ptr; }
-            *ptr = '\0';
-            if(strcmp(name, "rpl") == 0) { break; }
-            minor = -1;
-        }
-    } else {
-        fprintf(stderr, _("Could not open either /sys/module/rpldev/Minor_nr "
-         "or /proc/misc!\n"));
-        return NULL;
-    }
-
-    fclose(fp);
-
-    if(minor < 0) {
-        printf(_("rpldev module does not seem to be loaded or can not find\n"
-         "minor number associated with it.\n"));
-        return NULL;
-    }
-
-    unlink(NODEPATH); // safe side in case mknod() returns EEXIST
-    if(mknod(NODEPATH, S_IFCHR | S_IRUSR, COMPAT_MKDEV(10, minor)) != 0) {
-        perror("mknod()");
-        return NULL;
-    }
-
-    chown(NODEPATH, GOpt.user_id, 0);
-    return NODEPATH;
-}
-
 static int init_sighandler(void) {
     struct sigaction s_int, s_alrm, s_pipe;
 
@@ -558,10 +598,11 @@ static void sighandler_int(int s) {
 }
 
 static void sighandler_alrm(int s) {
-    fprintf(stderr, "\r\e[2K" "IOCD: %zu/%zu/%zu/%zu  RW: %zu/%zu (%llu/%llu)"
-      " I: %zu  B: %zu", Stats.init, Stats.open, Stats.close, Stats.deinit,
+    printf("\r\e[2K" "IOCD: %lu/%lu/%lu/%lu  RW: %lu/%lu (%llu/%llu)"
+      " I: %lu  B: %lu", Stats.init, Stats.open, Stats.close, Stats.deinit,
       Stats.read, Stats.write, Stats.in, Stats.out, Stats.ioctl, Stats.badpack
     );
+    fflush(stdout);
     if(GOpt.verbose) { alarm(1); }
     return;
 }
@@ -576,7 +617,7 @@ static int get_options(int *argc, const char ***argv) {
     static const char *_empty_argv[] = {NULL};
     char *tmp;
     struct poptOption options_table[] = {
-        {NULL, 'D', POPT_ARG_STRING, &Opt.ovdev, 0,
+        {NULL, 'D', POPT_ARG_STRING, &Opt.device, 0,
          "Path to the RPL device", "file"},
         {NULL, 'I', POPT_ARG_VAL, &Opt.infod_start, 1,
          "Start INFOD subcomponent", NULL},
@@ -639,11 +680,10 @@ static int get_options(int *argc, const char ***argv) {
 
 static int read_config(const char *file) {
     static struct rconfig_opt options_table[] = {
-        {"OVDEVICE", RCONF_STRING, &Opt.ovdev, NULL},
-        {"FBDEVICE", RCONF_STRING, &Opt.fbdev, NULL},
-        {"OFMT",     RCONF_STRING, &GOpt.ofmt, NULL},
-        {"DO_LOG",   RCONF_IBOOL,  &Opt.dolog, NULL},
-        {"USER",     RCONF_CB,     NULL,       pconfig_user},
+        {"DEVICE", RCONF_STRING, &Opt.device, NULL},
+        {"OFMT",   RCONF_STRING, &GOpt.ofmt,  NULL},
+        {"DO_LOG", RCONF_IBOOL,  &Opt.dolog,  NULL},
+        {"USER",   RCONF_CB,     NULL,        pconfig_user},
 
         {"INFOD_PORT", RCONF_STRING, &GOpt.infod_port, NULL},
         {"START_INFOD", RCONF_IBOOL, &Opt.infod_start, NULL},
@@ -682,10 +722,65 @@ inline static const char *basename(const char *s) {
     return s;
 }
 
+static int find_devnode(uint32_t id, char *dest, size_t len,
+ const char **loc_pbase)
+{
+    /* Walk through the list of directories containing all the device nodes and
+    compare their major/minor numbers with ID. Note that we will not search in
+    arbitrary locations, such as chroot jails. */
+    const char **dirp = Device_dirs;
+    while(*dirp != NULL) {
+        if(find_devnode_dive(id, dest, len, *dirp)) {
+            if(loc_pbase != NULL) { *loc_pbase = *dirp; }
+            return 1;
+        }
+        ++dirp;
+    }
+    return 0;
+}
+
+static int find_devnode_dive(uint32_t id, char *dest, size_t len,
+ const char *dir)
+{
+    /* Scan a directory for node. During directory traversal, everything that
+    begins in a dot is ignored -- this is crude behavior, but the simplicity
+    is justified given that there are no device nodes starting with a dot. And
+    if, you are either special or there is a good reason. */
+    char buf[MAXFNLEN];
+    struct stat sb;
+    void *dx;
+    char *de;
+
+    if((dx = HXdir_open(dir)) == NULL) { return 0; }
+    while((de = HXdir_read(dx)) != NULL) {
+        snprintf(buf, sizeof(buf), "%s/%s", dir, de);
+        if(*de == '.' || stat(buf, &sb) != 0) { continue; }
+        if(S_ISCHR(sb.st_mode) &&
+         K26_MKDEV(COMPAT_MAJOR(sb.st_rdev), COMPAT_MINOR(sb.st_rdev)) == id) {
+            strncpy(dest, buf, len);
+            dest[len - 1] = '\0';
+            HXdir_close(dx);
+            return 1;
+        }
+        if(S_ISDIR(sb.st_mode)) {
+            snprintf(buf, sizeof(buf), "%s/%s", dir, de);
+            if(find_devnode_dive(id, dest, len, buf)) { return 1; }
+        }
+    }
+    HXdir_close(dx);
+    return 0;
+}
+
 static char *getnamefromuid(uid_t uid, char *result, size_t len) {
-    struct passwd ent, *ep;
+    // Turn a UID into a username, if possible.
+    struct passwd *ep;
+#if defined(__OpenBSD__)
+    ep = getpwuid(uid);
+#else
+    struct passwd ent;
     char additional[1024];
     getpwuid_r(uid, &ent, additional, sizeof(additional), &ep);
+#endif
     if(ep == NULL) { return NULL; }
     strncpy(result, ep->pw_name, len - 1);
     result[len - 1] = '\0';
@@ -693,9 +788,14 @@ static char *getnamefromuid(uid_t uid, char *result, size_t len) {
 }
 
 static uid_t getuidfromname(const char *name) {
-    struct passwd ent, *ep;
+    struct passwd *ep;
+#if defined(__OpenBSD__)
+    ep = getpwnam(name);
+#else
+    struct passwd ent;
     char additional[1024];
     getpwnam_r(name, &ent, additional, sizeof(additional), &ep);
+#endif
     if(ep == NULL) { return -1; }
     return ep->pw_uid;
 }
